<?php
namespace Service;

use Util\Utilities;
use Zeedhi\Framework\Session\Session;

class SocketClient {

    const NSP        = 'ZmartDataCollector';
    const SERVER_KEY = 'you can create!';

    // Events
    const CREATE_NSP                = 'createNamespace';
    const CREATE_ROOM               = 'createRoom';
    const JOIN_ROOM                 = 'joinRoom';
    const EMIT                      = 'emit';
    const ON                        = 'on';
    const START_IMPORTATION         = 'startImportation';
    const NOTIFY_ERROR              = 'onError';
    const ERROR_SUFFIX 			    = "Error";
    const RESPONSE_SUFFIX 		    = "Response";
    const GET_COLUMNS_META_DATA     = 'getColumnsMetaData';
    const EXEC_QUERY                = 'execQuery';
    const TEST_CONN                 = 'testConnection';
    const NO_CLIENTS_CONNECTED      = 'noClientsConnected';
    const JOIN_NEW_ROOM             = 'joinNewRoom';
    const GET_LOG_LIST              = 'getLogList';
    const GET_LOG_CONTENT           = 'getLogContent';

    protected $client;
    protected $isConnected = false;

    protected $timeout;

    public function __construct(\WebSocket\Client $client, $options) {
        $this->timeout = $options['timeout'];
        $this->client = $client;
    }

    protected function send(array $payload) {
        $this->client->send(json_encode($payload));
    }

    protected function connectToZmartNamespace() {
        if (!$this->isConnected) {
            $this->send($this->buildBasePayload(self::CREATE_NSP, null, array('key' => self::SERVER_KEY)));
            $this->isConnected = true;
        }
    }

    protected function joinRoom($room, $create = false) {
        if($create) {
            $this->createRoom($room);
        }
        $this->send($this->buildBasePayload(self::JOIN_ROOM, $room));
    }

    protected function createRoom($room) {
        $this->send($this->buildBasePayload(self::CREATE_ROOM, $room));
    }

    protected function buildBasePayload($event, $room = null, $data = array()) {
        return array(
            'event'     => $event,
            'namespace' => self::NSP,
            'room'      => $room,
            'data'      => $data
        );
    }

    protected function subscribeToEvent($room, $eventName) {
        $this->send($this->buildBasePayload(self::ON, $room, array('event' => $eventName)));
    }

    protected function emitEvent($room, $data) {
        $event = $this->buildBasePayload(self::EMIT, $room, $data);
        $this->send($event);
    }

    public function getColumnsMetaData($query, $room, $maxWaitTime = null, $connectionParams) {
        $data = $this->getDataFromParams(self::GET_COLUMNS_META_DATA, $query, $connectionParams);
        return $this->executeCommandOnDefaultRoom($room, self::GET_COLUMNS_META_DATA, $data, $maxWaitTime);
    }

    protected function executeCommandOnDefaultRoom($room, $event, $data = array(), $maxWaitTime = null) {
        $this->connectToZmartNamespace();

        $this->createNewRoom($room, $maxWaitTime);
        $this->subscribeToEvent($room, $event.self::RESPONSE_SUFFIX);
        $this->subscribeToEvent($room, $event.self::ERROR_SUFFIX);

        $this->emitEvent($room, $data);

        $res = $this->waitForResponseSync(array($event), $maxWaitTime, true);

        return $res;
    }

    protected function executeCommandOnTempRoom($room, $command, callable $callback, $maxWaitTime = null) {
        $this->connectToZmartNamespace();

        $newRoom = $this->createNewRoom($room, $maxWaitTime);

        $this->subscribeToEvent($newRoom, $command.self::RESPONSE_SUFFIX);
        $this->subscribeToEvent($newRoom, $command.self::ERROR_SUFFIX);

        $callback($newRoom);

        return $this->waitForResponseSync(array($command), $maxWaitTime, true);
    }

    public function execQuery($query, $room, $maxWaitTime = null, $connectionParams) {
        return $this->executeCommandOnTempRoom($room, self::EXEC_QUERY, function($newRoom) use ($query, $connectionParams) {
            $data = $this->getDataFromParams(self::EXEC_QUERY, $query, $connectionParams);
            $data['newRoom'] = $newRoom;
            $this->emitEvent($newRoom, $data);
        }, $maxWaitTime);
    }

    public function getLogList($room) {
        return $this->executeCommandOnTempRoom($room, self::GET_LOG_LIST, function($newRoom) {
            $data = array(
                'newRoom' => $newRoom,
                'event' => self::GET_LOG_LIST
            );

            $this->emitEvent($newRoom, $data);
        });
    }

    public function getLogContent($room, $logFile) {
        return $this->executeCommandOnTempRoom($room, self::GET_LOG_CONTENT, function($newRoom) use ($logFile) {
            $data = array(
                'newRoom' => $newRoom,
                'query' => $logFile,
                'event' => self::GET_LOG_CONTENT
            );

            $this->emitEvent($newRoom, $data);
        });
    }

    public function testConnection($room, $connectionParams, $query = null, $maxWaitTime = null) {
        $data = $this->getDataFromParams(self::TEST_CONN, $query, $connectionParams);
        $response = $this->executeCommandOnDefaultRoom($room, self::TEST_CONN, $data, $maxWaitTime);
        return empty($response) ? array() : $response[0];
    }

    protected function createNewRoom($room, $maxWaitTime = null) {
        $this->joinRoom($room, true);
        $newRoom = uniqid($room.'_');
        $data = array(
            "event" => self::JOIN_NEW_ROOM,
            "newRoom" => $newRoom
        );

        $this->emitEvent($room, $data);
        $this->joinRoom($newRoom, true);
        $this->subscribeToEvent($newRoom, self::JOIN_NEW_ROOM.self::RESPONSE_SUFFIX);
        $this->subscribeToEvent($newRoom, self::JOIN_NEW_ROOM.self::ERROR_SUFFIX);
        $response = $this->waitForResponseSync(array(self::JOIN_NEW_ROOM), $maxWaitTime, true);
        $response = $response && !isset($response['hasError']) ? $newRoom : null;

        return $newRoom;
    }

    protected function getDataFromParams($event, $query, $params){
        return array(
            "event"       => $event,
            "query"       => $query,
            "userNameSsl" => isset($params["userNameSsl"]) ? $params["userNameSsl"] : null,
            "passwordSsl" => isset($params["passwordSsl"]) ? $params["passwordSsl"] : null,
            "dbUrlSsl" 	  => isset($params["dbUrlSsl"])	   ? $params["dbUrlSsl"]    : null,
            "userName"    => isset($params['userName'])    ? $params['userName']    : null,
            "password"    => isset($params['password'])    ? $params['password']    : null,
            "dbUrl"       => isset($params['dbUrl'])       ? $params['dbUrl']       : null,
            "dialect"     => isset($params['dialect'])     ? $params['dialect']     : null,
            "driverClass" => isset($params['driverClass']) ? $params['driverClass'] : null,
            "oracleWalletUse" => isset($params['oracleWalletUse']) ? $params['oracleWalletUse'] : null
        );
    }

    protected function waitForResponseSync(array $eventsToWait, $maxWaitTime = null, $stopOnFirstResponse = false) {
        // Sets a huge value of time limit to accept big timeouts
        set_time_limit(600);
        if ($maxWaitTime === null) {
            $maxWaitTime = $this->timeout;
        }
        $counter = 0;
        $response = array();

        $startTime = time();
        $shouldStopWaiting = false;
        while (time() < $startTime+$maxWaitTime && !$shouldStopWaiting) {
            try {
                $treatedRes = $this->treatResponsePayload($this->client->receive());

                if (empty($eventsToWait) || $this->checkSubscribedEvents($eventsToWait, $treatedRes['event'])) {
                    $response[] = $treatedRes;
                    if ($stopOnFirstResponse) {
                        $shouldStopWaiting = true;
                    }
                }
            } catch(\WebSocket\ConnectionException $e) {
                //Any connection errors will be ignored
                //If all iterations are connection errors, the result will be a timeout
            } finally {
                $counter++;
            }
            sleep(1);
        }

        if (empty($response)) {
            $response[] = array (
                'error' => 'Maximum timeout reached',
                'hasError' => true
            );
        }

        return $response;
    }

    protected function treatResponsePayload($response) {
        $treatedRes = json_decode($response, true);
        if ($treatedRes['event'] === self::NO_CLIENTS_CONNECTED) {
            $treatedRes['error'] = 'CLIENT_DATABASE_CONNECT_FAIL';
            $treatedRes['hasError'] = true;
        } else if (Utilities::endsWith($treatedRes['event'], self::ERROR_SUFFIX)) {
            $treatedRes['error'] = $this->setErrorMessageBasedOnEvent($treatedRes);
            $treatedRes['hasError'] = true;
        }

        return $treatedRes;
    }

    protected function checkSubscribedEvents($eventsToWait, $eventToCheck) {
        $result = false;

        foreach ($eventsToWait as $event) {
            $isResponseEvent = $eventToCheck === ($event . self::RESPONSE_SUFFIX);
            $isErrorEvent = $eventToCheck === ($event . self::ERROR_SUFFIX);
            $isNoClientsConnectedEvent = $eventToCheck === self::NO_CLIENTS_CONNECTED;
            if ($event === $eventToCheck || $isNoClientsConnectedEvent || $isResponseEvent || $isErrorEvent) {
                $result = true;
            }
        }

        return $result;
    }

    protected function setErrorMessageBasedOnEvent($res) {
        $event = $res['event'];
        $data  = isset($res['data'])? $res['data'] : array();
        $message = '';

        switch (true) {
            case Utilities::startsWith($event, self::GET_COLUMNS_META_DATA):
                $dataResponse = isset($data['response'])? $data['response'] : array();
                $message = isset($dataResponse['detailMessage'])? $dataResponse['detailMessage'] : 'Error while loading columns from query';
                break;
            case Utilities::startsWith($event, self::EXEC_QUERY):
                $dataResponse = isset($data['response'])? $data['response'] : array();
                $message = isset($dataResponse['detailMessage'])? $dataResponse['detailMessage'] : 'Error while executing query';
                break;
            default:
                $message = 'Error while making the transaction';
                break;
        }

        return $message;
    }

    public static function factory($socketServerAddress, $options = array()) {
        if (empty($options['timeout'])) {
            $options['timeout'] = 50;
        }
        return new SocketClient(new \WebSocket\Client($socketServerAddress, $options), $options);
    }

    public function startImportation($organization, $process, $query, $queryKeys, $pageSize, $dbUrl, $userName, $password,
                                     $scriptBefore, $scriptAfter, $execAfterIfImportError, $execAfterIfBeforeScriptError,
                                     $clientRoom, $dialect, $driverClass, $usernameSsl, $passwordSsl, $dbUrlSsl) {
        $this->joinRoom($clientRoom);

        $data = array(
            'organizationId'               => $organization,
            'event'                        => self::START_IMPORTATION,
            'process'                      => $process,
            'query'                        => $query,
            'queryKeys'                    => isset($queryKeys) ? $queryKeys : array(),
            'pageSize'                     => $pageSize,
            'dbUrl'                        => $dbUrl,
            'userName'                     => $userName,
            'password'                     => $password,
            'dbUrlSsl'                     => $dbUrlSsl,
            'userNameSsl'                  => $usernameSsl,
            'passwordSsl'                  => $passwordSsl,
            'scriptBefore'                 => $scriptBefore,
            'scriptAfter'                  => $scriptAfter,
            'execAfterIfImportError'       => $execAfterIfImportError,
            'execAfterIfBeforeScriptError' => $execAfterIfBeforeScriptError,
            'driverClass'                  => $driverClass,
            'dialect'                      => $dialect
        );
        $this->send($this->buildBasePayload(self::EMIT, $clientRoom, $data));
    }
}
