
// FILE: src/interval/config/config.js
window.templateUrls = window.templateUrls || [];
var packageName = 'zh-interval-component';

var hasPackageRegistered = window.templateUrls.some(function(templateUrl){
	return templateUrl.package === packageName;
});

if(!hasPackageRegistered){
	window.templateUrls.push({
		'package': packageName,
		'baseUrl': './bower_components/' + packageName + '/dist/templates/'
	});
}

// FILE: src/interval/registers/IntervalRegister.js
function IntervalRegister(DecodeValueFieldService, validateEngine, ScreenService) {
    var self = this;

    /**
     * DecodeValueField Register
     **/
    var DEFAULT_DISPLAY_FORMAT = 'YYYY-MM-DD';
    var DEFAULT_DATE_FORMAT = 'YYYY-MM-DD';

    self.decodeValueField = function(field, fieldRow) {
        var displayValue = "";
        var dateFormat = field.dateFormat || DEFAULT_DATE_FORMAT;
        var formattedDate = field.displayFormat || DEFAULT_DISPLAY_FORMAT;
        if(fieldRow){
            if (fieldRow.end || fieldRow.start || fieldRow.period) {
                var beginDate = fieldRow.start ? moment(fieldRow.start, dateFormat).format(formattedDate) : formattedDate.replace(/[a-z]/gi, '_');
                var endDate = fieldRow.end ? moment(fieldRow.end, dateFormat).format(formattedDate) : formattedDate.replace(/[a-z]/gi, '_');
                if (!fieldRow.period) {
                    displayValue = ScreenService.i18n("From ") + beginDate + ScreenService.i18n(" to ") + endDate;
                    return displayValue;
                } else {
                    var periodLabel = fieldRow.period.label;
                    displayValue = ScreenService.i18n("From ") + beginDate + ScreenService.i18n(" to ") + endDate + ' (' + periodLabel + ') ';
                    return displayValue;
                }

            }
        }

        return ScreenService.i18n("From ") + formattedDate.replace(/[a-z]/gi, '_') + ScreenService.i18n(" to ") + formattedDate.replace(/[a-z]/gi, '_');
    };

    DecodeValueFieldService.register("interval.html", self.decodeValueField);

    /**
     * ValidateEngine Register
     **/
    function required(value, validationAttrs) {
        return {
            "valid": requiredStart(value, validationAttrs).valid && requiredEnd(value, validationAttrs).valid,
            "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Required field")
        };
    }

    function requiredStart(value, validationAttrs) {
        return buildRequiredMessage(value, validationAttrs, 'start');   
    }

    function requiredEnd(value, validationAttrs) {
        return buildRequiredMessage(value, validationAttrs, 'end');
    }

    function buildRequiredMessage(value, validationAttrs, property) {
        return {
            "valid": value && Util.isObject(value) && !Util.isEmptyOrBlank(value[property]),
            "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n('Required field')
        };
    }

    var registeredValidations = validateEngine.getMappedValidations();
    var validationsToRegister = {
        "requiredInterval": required,
        "requiredIntervalStart": requiredStart,
        "requiredIntervalEnd": requiredEnd
    };

    for (var validationName in validationsToRegister) {
        if (!registeredValidations.hasOwnProperty(validationName)) {
            validateEngine.registerValidation(validationName, validationsToRegister[validationName]);
        }
    }
}

Configuration(IntervalRegister);

// FILE: src/interval/controllers/IntervalController.js
function IntervalController($scope, ZhUtil, ScreenService, validateEngine) {

	var self = this;
	self.field = $scope.field;
	self.widget = $scope.widget;

	/**
	 * Constant values
	 **/
	var DEFAULT_DISPLAY_FORMAT = 'YYYY-MM-DD';
	var DEFAULT_DATE_FORMAT = 'YYYY-MM-DD';
	var DEFAULT_START_LABEL = 'Begin on';
	var DEFAULT_END_LABEL = 'End on';
	var INCLUSIVE_SUFFIX = "inclusive";

	/**
	 * Initial values
	 **/
	var sourceElement = self.field || self.widget;
	$scope.dateFormat = sourceElement.dateFormat || DEFAULT_DATE_FORMAT;
	$scope.displayFormat = sourceElement.displayFormat || DEFAULT_DISPLAY_FORMAT;
	$scope.startLabel = ScreenService.i18n(sourceElement.startLabel || DEFAULT_START_LABEL);
	$scope.endLabel = ScreenService.i18n(sourceElement.endLabel || DEFAULT_END_LABEL);

	self.formatDateForShowOnField = function(property) {
		var valueOnCurrentRow = self.field.widget.currentRow[self.field.name]; //Generate mask type like __/__/____
		var formattedDate = moment().format($scope.displayFormat).replace(/[0-9]/g, '_');
		if (valueOnCurrentRow && (property === 'start' || property === 'end') && valueOnCurrentRow[property]) {
			formattedDate = moment(valueOnCurrentRow[property], $scope.dateFormat).format($scope.displayFormat);
		}
		return formattedDate;
	};

	function buildIntervals() {
		var fieldIntervals = self.field.intervals;
		var finalIntervals = [];
		
		//Set default Intervals 
		if (fieldIntervals === undefined) {
			fieldIntervals = [{
				label: "Today",
				period: "today"
			}, {
				label: "Yesterday",
				period: "yesterday"
			}, {
				label: "Tomorrow",
				period: "tomorrow"
			}, {
				label: "This Week",
				period: "this-week"
			}, {
				label: "Last Week",
				period: "last-week"
			}, {
				label: "Next Week",
				period: "next-week"
			}, {
				label: "This Quarter",
				period: "this-quarter"
			}, {
				label: "Last Quarter",
				period: "last-quarter"
			}, {
				label: "Next Quarter",
				period: "next-quarter"
			}, {
				label: "This Month",
				period: "this-month"
			}, {
				label: "Last Month",
				period: "last-month"
			}, {
				label: "Next Month",
				period: "next-month"
			}, {
				label: "This Year",
				period: "this-year"
			}, {
				label: "Last Year",
				period: "last-year"
			}, {
				label: "Next Year",
				period: "next-year"
			}];	
		}

		// Iterates in field periods to build each determined row
		if (Util.isArray(fieldIntervals)) {
			fieldIntervals.forEach(function(interval) {
				try {
					/**
					 *   @Note:
					 * - The customized flag is used to tell that such period was created by developer.
					 * - The personalized flag is used to tell that such period will be selected by the user because it's not in the period list.
					 */

					var row;

					// Evaluates possible custom intervals
					if (isCustomPeriod(interval.period)) {
						row = ZhUtil.evaluate(interval.period);
						row.customized = true;
					}
					else {
						row = buildObjectWithPeriodDetails(interval.period);
					}

					finalIntervals.push({
						"_period_label": interval.label,
						"_period": row,
						"_period_name": interval.period
					});
				}
				catch (e) {}
			});
		}

		return finalIntervals;
	}

	function isCustomPeriod(periodName) {
		return periodName[0] === '@';
	}

	function buildObjectWithPeriodDetails(word) {
		var decoded = decodeWordExpression(word);
		var buildIntervalMethod = (decoded.suffix === INCLUSIVE_SUFFIX) ? buildIntervalInclusive : buildInterval;
		return buildIntervalMethod(decoded.prefix, decoded.amount, decoded.unit);
	}

	function decodeWordExpression(word) {
		var exceptionWords = {
			'today': 'this-day',
			'yesterday': 'last-day',
			'tomorrow': 'next-day'
		};

		// Transform exception words due to function needs
		if (word in exceptionWords) {
			word = exceptionWords[word];
		}

		var prefix, amount, unit, suffix;
		var splitedString = word.split("-");

		if (splitedString.length === 2) {
			prefix = splitedString[0];
			unit = splitedString[1];
		}
		else if (splitedString.length > 2) {
			prefix = splitedString[0];
			amount = splitedString[1];
			unit = splitedString[2];
			suffix = splitedString[3];
		}
		else {
			throw "Used period has an incorrect format.";
		}

		return {
			prefix: prefix,
			amount: amount || 1,
			unit: unit,
			suffix: suffix
		};
	}

	function buildInterval(prefix, amount, unit) {
		var start = moment().startOf(unit),
			end = moment();
		switch (prefix) {
			case 'last':
				end.subtract(1, unit);
				start.subtract(amount, unit);
				break;
			case 'next':
				start.add(1, unit);
				end.add(amount, unit);
				break;
		}
		end.endOf(unit);
		return factoryInterval(start, end);
	}

	function buildIntervalInclusive(prefix, amount, unit) {
		var start = moment().startOf(unit),
			end = moment();
		switch (prefix) {
			case 'last':
				start.subtract(amount - 1, unit);
				break;
			case 'next':
				end.add(amount - 1, unit);
				break;
		}
		end.endOf(unit);
		return factoryInterval(start, end);
	}

	function factoryInterval(start, end) {
		return {
			"start": start.format($scope.dateFormat),
			"end": end.format($scope.dateFormat),
			"customized": false
		};
	}

	function clearInternalValues() {
		var intervalField = self.field;
		var parentWidget = intervalField.widget;
		delete parentWidget.currentRow[intervalField.name];
		intervalField.validate(intervalField.value());
		if (self.field.change) {
			self.field.change();
		}
	}

	function createIntervalWidget() {
		var actions = [{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"name": "ActionEvent",
				"code": "args.owner.widget.parent.closeIntervalPopup(args.owner)",
				"id": 0
			}]
		}, {
			"label": ScreenService.i18n("OK"),
			"showAsAction": "never",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"events": [{
				"name": "ActionEvent",
				"code": "args.owner.widget.parent.closeIntervalPopup(args.owner); args.owner.widget.parent.applyIntervalToWidgetRow();",
				"id": 1
			}]
		}];

		return metaDataFactory.widgetFactory({
			"id": "0",
			"name": "Interval",
			"label": self.field.label,
			"template": "zh-interval-component#widget/interval.html",
			"container": self.field.widget.container,
			"isVisible": true,
			"popupNoBlock": false,
			"startLabel": self.field.startLabel,
			"endLabel": self.field.endLabel,
			"dateFormat": $scope.dateFormat,
			"displayFormat": self.field.displayFormat || $scope.dateFormat,
			"widgets": [{
				"id": "0.1",
				"name": "Calendar",
				"label": self.field.label,
				"template": "widget/calendar.html",
				"container": self.field.widget.container,
				"parent": "",
				"dateFormat": $scope.dateFormat,
				"displayFormat": self.field.displayFormat || $scope.dateFormat,
				"isVisible": true,
				"floatingControl": false,
				"dataSource": {
					"data": self.field.dataSource
				},
				"showDescriptionOrder": false,
				"events": [{
					"name": "WidgetAfterMoveRow",
					"code": "args.owner.parent.selectIntervalDate(args)"
				}, {
					"name": "WidgetOnEnter",
					"code": "args.owner.parent.selectDatesBasedOnCurrentRow()"
				}],
				"actions": actions
			}, {
				"id": "0.2",
				"name": "List",
				"label": self.field.label,
				"template": "widget/list.html",
				"container": self.field.widget.container,
				"parent": "",
				"isVisible": true,
				"showDescriptionOrder": false,
				"floatingControl": false,
				"fields": [{
					"label": "Period",
					"name": "_period_label",
					"showOnList": true
				}],
				"dataSource": {
					"data": buildIntervals()
				},
				"events": [{
					"name": "WidgetAfterMoveRow",
					"code": "args.owner.parent.selectIntervalPeriod(args.owner)"
				}],
				"actions": actions
			}],
			"events": []
		});
	}

	this.openIntervalWidget = function() {
		self.field.intervalWidget = createIntervalWidget();
		applyIntervalWidgetInitialValues();
		resetIntervalValuesOnCurrentRow();
		createIntervalWidgetMethods();
		ScreenService.openPopup(self.field.intervalWidget);
	};

	function applyIntervalWidgetInitialValues() {
		var intervalWidget = self.field.intervalWidget;
		intervalWidget.isIntervalEmpty = true; // little flag
		intervalWidget.beingSelected = 'start';
		intervalWidget.setBeingSelected = setBeingSelected;
		intervalWidget.carouselWidget = 'calendar';

		intervalWidget.intervalData = {};
		intervalWidget.intervalData.start = getInitialstartDate();
		intervalWidget.intervalData.end = getInitialEndDate();
		intervalWidget.widgets[0].currentDate = intervalWidget.intervalData.start;
		intervalWidget.formattedData = {};
		intervalWidget.formattedData.start = moment(getInitialstartDate(), $scope.dateFormat).format($scope.displayFormat);
		intervalWidget.formattedData.end = moment(getInitialEndDate(), $scope.dateFormat).format($scope.displayFormat);
	}

	function getInitialstartDate() {
		return getInitialDate('start');
	}

	function getInitialEndDate() {
		return getInitialDate('end');
	}

	function getInitialDate(property) {
		var field = self.field;
		var parentWidget = field.widget;
		var currentRowProperty = parentWidget.currentRow[field.name];
		return currentRowProperty ? currentRowProperty[property] : moment().format($scope.dateFormat);
	}

	function resetIntervalValuesOnCurrentRow() {
		var intervalField = self.field;
		var parentWidget = intervalField.widget;
		if (!parentWidget.currentRow[intervalField.name]) {
			parentWidget.currentRow[intervalField.name] = {
				start: null,
				end: null,
				period: null
			};
		}
	}

	function setBeingSelected(value) {
		self.field.intervalWidget.beingSelected = value;
	}

	function createIntervalWidgetMethods() {
		self.field.intervalWidget.applyIntervalToWidgetRow = function() {
			var intervalField = self.field;
			var parentWidget = intervalField.widget;
			var intervalData = intervalField.intervalWidget.intervalData;

			parentWidget.currentRow[intervalField.name] = {
				start: intervalData.start,
				end: intervalData.end,
				period: intervalData.period
			};
			intervalField.validate(intervalField.value());
			if (intervalField.change) {
				intervalField.change();
			}
		};

		self.field.intervalWidget.selectDatesBasedOnCurrentRow = function() {
			var field = self.field;
			var currentRow = field.widget.currentRow;
			var intervalWidget = field.intervalWidget;
			var intervalData = intervalWidget.intervalData;
			var formattedData = intervalWidget.formattedData;
			var startDate = currentRow[field.name] ? currentRow[field.name].start : null;
			var endDate = currentRow[field.name] ? currentRow[field.name].end : null;

			if (Util.isValidDate(startDate, $scope.dateFormat) && Util.isValidDate(endDate, $scope.dateFormat)) {
				intervalData.start = startDate;
				intervalData.end = endDate;
				formattedData.start = moment(startDate, $scope.dateFormat).format($scope.displayFormat);
				formattedData.end = moment(endDate, $scope.dateFormat).format($scope.displayFormat);
				selectIntervalOnCalendar();
			}
			else {
				intervalData.start = moment().format($scope.dateFormat);
				intervalData.end = moment().format($scope.dateFormat);
				formattedData.start = moment().format($scope.displayFormat);
				formattedData.end = moment().format($scope.displayFormat);
				selectIntervalOnCalendar();
			}
		};

		self.field.intervalWidget.selectIntervalDate = function(args) {
			var intervalWidget = self.field.intervalWidget;
			var intervalData = intervalWidget.intervalData;
			var formattedData = intervalWidget.formattedData;
			var selectedDate = args.row;
			var momentSelectedDate = moment(args.row, $scope.dateFormat);
			var momentstartDate = moment(intervalData.start, $scope.dateFormat);
			var momentEndDate = moment(intervalData.end, $scope.dateFormat);

			if (intervalWidget.beingSelected !== 'start' &&
				intervalWidget.beingSelected !== 'end'
			) {
				intervalWidget.beingSelected = 'start';
			}

			intervalData[intervalWidget.beingSelected] = selectedDate;
			formattedData[intervalWidget.beingSelected] = moment(selectedDate, $scope.dateFormat).format($scope.displayFormat);

			var shouldToggleBeingSelected = true;

			if (intervalWidget.beingSelected === 'start' &&
				momentSelectedDate.isAfter(momentEndDate)
			) {
				intervalData.end = selectedDate;
			}
			else if (intervalWidget.beingSelected === 'end' &&
				momentSelectedDate.isBefore(momentstartDate)
			) {
				shouldToggleBeingSelected = false;
				intervalData.start = selectedDate;
			}

			if (shouldToggleBeingSelected) {
				toggleBeingSelected();
			}

			clearOldInterval();
			selectIntervalOnCalendar();
			intervalWidget.isIntervalEmpty = false;
		};

		self.field.intervalWidget.closeIntervalPopup = function(owner) {
			ScreenService.closePopup();
			var intervalField = self.field;
			var toBeActivatedWidget = this.lastActiveWidget;
			var intervalPeriod = toBeActivatedWidget.currentRow[intervalField.name];

			if (intervalPeriod === undefined) {
				getLastActivatedWidget();
			}
			else if (intervalPeriod.start === null && intervalPeriod.end === null && intervalPeriod.period === null) {
				delete toBeActivatedWidget.currentRow[intervalField.name];
			}

			getLastActivatedWidget();
		};

		self.field.intervalWidget.selectIntervalPeriod = function(listWidget) {
			var intervalWidget = self.field.intervalWidget;
			var intervalData = intervalWidget.intervalData;
			intervalData.start = listWidget.currentRow._period.start;
			intervalData.end = listWidget.currentRow._period.end;
			intervalData.period = {
				'name': listWidget.currentRow._period_name,
				'label': listWidget.currentRow._period_label,
				'customized': listWidget.currentRow._period.customized
			};

			intervalWidget.formattedData.start = moment(intervalData.start, $scope.dateFormat).format($scope.displayFormat);
			intervalWidget.formattedData.end = moment(intervalData.end, $scope.dateFormat).format($scope.displayFormat);

			clearOldInterval();
			selectIntervalOnCalendar();

			ScreenService.closePopup();
			self.field.widget.activate();
			intervalWidget.applyIntervalToWidgetRow();
		};

		self.field.intervalWidget.isCalendarCurrentWidget = function() {
			return self.field.intervalWidget.carouselWidget === 'calendar';
		};

		self.field.intervalWidget.isPeriodSelectorCurrentWidget = function() {
			return self.field.intervalWidget.carouselWidget === 'periodSelector';
		};

		self.field.intervalWidget.toggleCurrentWidget = function() {
			var intervalWidget = self.field.intervalWidget;
			if (intervalWidget.isPeriodSelectorCurrentWidget()) {
				intervalWidget.carouselWidget = 'calendar';
				intervalWidget.beingSelected = 'start';
			}
			else {
				intervalWidget.carouselWidget = 'periodSelector';
				intervalWidget.beingSelected = 'none';
			}

			getLastActivatedWidget();
			intervalWidget.nextWidget();
		};
	}


	function getLastActivatedWidget() {
		var lastActive = self.field.intervalWidget.lastActiveWidget;
		var toBeActivatedWidget = self.field.intervalWidget.lastActiveWidget;
		toBeActivatedWidget.activate();
		toBeActivatedWidget.lastActiveWidget = lastActive;
	}

	function toggleBeingSelected() {
		var intervalWidget = self.field.intervalWidget;
		intervalWidget.beingSelected = intervalWidget.beingSelected === 'start' ? 'end' : 'start';
	}

	function clearOldInterval() {
		var calendarWidget = self.field.intervalWidget.widgets[0];
		calendarWidget.selectedDates = [];
	}

	function selectIntervalOnCalendar() {
		var intervalWidget = self.field.intervalWidget || self.widget;
		var calendarWidget = intervalWidget.widgets[0];

		var dateFormat = self.field.dateFormat;
		var startDate = intervalWidget.intervalData.start;
		var endDate = intervalWidget.intervalData.end;

		var momentstartDate = moment(startDate, dateFormat);
		var momentEndDate = moment(endDate, dateFormat);
		var momentCurrentDate = momentstartDate;

		while (momentCurrentDate.isBefore(momentEndDate) || momentCurrentDate.isSame(momentEndDate)) {
			calendarWidget.setDateAsSelected(momentCurrentDate.format(dateFormat), dateFormat);
			momentCurrentDate.add(1, 'day');
		}

	}

	this.getPeriodLabel = function() {
		var label = '';
		var property = self.field.widget.currentRow[self.field.name];
		if (property && property.period && property.period.label) {
			label = '(' + property.period.label + ')';
		}
		return label;
	};

	if (self.field) {
		var field = self.field;
		var parentWidget = field.widget;

		field.value(self.field.value() || self.field.defaultValue);
		field.isRowSet = function() {
			var currentRow = this.widget.currentRow;
			return currentRow[this.name] && currentRow[this.name].start && currentRow[this.name].end;
		};
		field.clearValue = clearInternalValues;

		field.getDatesByPeriodName = function(periodName, periodLabel) {
			periodLabel = periodLabel || '';
			var date = isCustomPeriod(periodName) ? ZhUtil.evaluate(periodName) : buildObjectWithPeriodDetails(periodName);

			return {
				"start": date.start,
				"end": date.end,
				"period": {
					"name": periodName,
					"label": periodLabel || '',
					"customized": date.customized
				}
			};
		};

		if (parentWidget.currentRow[field.name] &&
			parentWidget.currentRow[field.name].period &&
			parentWidget.currentRow[field.name].period.name
		) {
			var date = field.getDatesByPeriodName(parentWidget.currentRow[field.name].period.name);
			parentWidget.currentRow[field.name].start = date.start;
			parentWidget.currentRow[field.name].end = date.end;
		}
	}

	/**
	 * Validations
	 **/
	$scope.isRequired = function(field) {
		return field.validations && (field.validations.requiredInterval || field.validations.requiredIntervalStart || field.validations.requiredIntervalEnd);
	};
}